use gtk4 as gtk;
use gtk::{glib, prelude::*, subclass::prelude::*};
use crate::application;

mod imp {
    use super::*;
    use std::cell::RefCell;
    
    #[derive(Debug, Default, gtk::CompositeTemplate, glib::Properties)]
    #[template(file = "data/main_panel.blp")]
    #[properties(wrapper_type = super::MainPanel)]
    pub struct MainPanel {
        #[template_child]
        pub overlay: TemplateChild<gtk::Overlay>,
        #[template_child]
        pub search_box: TemplateChild<gtk::SearchEntry>,
        #[template_child]
        pub view_dropdown: TemplateChild<gtk::DropDown>,
        #[template_child]
        pub search_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub new_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub edit_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub delete_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub prev_page_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub page_label: TemplateChild<gtk::Entry>,
        #[template_child]
        pub next_page_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub scroll_overlay: TemplateChild<gtk::Overlay>,
        #[template_child]
        pub alias_list_view: TemplateChild<gtk::ListView>,
        #[template_child]
        pub alias_item_factory: TemplateChild<gtk::SignalListItemFactory>,
        #[template_child]
        pub alias_selection_model: TemplateChild<gtk::SingleSelection>,
        #[template_child]
        pub no_content_label: TemplateChild<gtk::Label>,
        #[template_child]
        pub error_overlay: TemplateChild<gtk::Box>,
        #[template_child]
        pub error_refresh_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub error_close_button: TemplateChild<gtk::Button>,
        #[template_child]
        pub error_label: TemplateChild<gtk::Label>,
        #[template_child]
        pub loading_overlay: TemplateChild<gtk::Box>,
        #[property(get, construct_only)]
        pub application: RefCell<application::Application>,
    }

    #[gtk::template_callbacks]
    impl MainPanel {}

    #[glib::object_subclass]
    impl ObjectSubclass for MainPanel {
        const NAME: &'static str = "MainPanel";
        type Type = super::MainPanel;
        type ParentType = gtk::Box;

        fn class_init(class: &mut Self::Class) {
            class.bind_template();
            class.bind_template_callbacks();
        }

        fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
            obj.init_template();
        }
    }

    #[glib::derived_properties]
    impl ObjectImpl for MainPanel {}
    impl BoxImpl for MainPanel {}
    impl WidgetImpl for MainPanel {}
}

glib::wrapper! {
    pub struct MainPanel(ObjectSubclass<imp::MainPanel>)
        @extends gtk::Widget, gtk::Box, @implements gtk::Accessible,
        gtk::Buildable, gtk::ConstraintTarget, gtk::Orientable;
}

impl MainPanel {
    pub fn new(app: &application::Application) -> Self {
        glib::Object::builder().property("application", app).build()
    }
}
